export class deviceBase {
    platform;
    accessory;
    location;
    device;
    sensorAccessory;
    group;
    api;
    log;
    config;
    hap;
    // Config
    deviceLogging;
    deviceRefreshRate;
    devicePushRate;
    deviceMaxRetries;
    deviceDelayBetweenRetries;
    constructor(platform, accessory, location, device, sensorAccessory, group) {
        this.platform = platform;
        this.accessory = accessory;
        this.location = location;
        this.device = device;
        this.sensorAccessory = sensorAccessory;
        this.group = group;
        this.api = this.platform.api;
        this.log = this.platform.log;
        this.config = this.platform.config;
        this.hap = this.api.hap;
        this.getDeviceLogSettings(device);
        this.getDeviceRateSettings(device);
        this.getDeviceRetry(device);
        this.getDeviceConfigSettings(device);
        this.getDeviceContext(accessory, device, sensorAccessory);
        // Set accessory information
        accessory
            .getService(this.hap.Service.AccessoryInformation)
            .setCharacteristic(this.hap.Characteristic.Manufacturer, 'Resideo')
            .setCharacteristic(this.hap.Characteristic.Name, accessory.context.name)
            .setCharacteristic(this.hap.Characteristic.ConfiguredName, accessory.context.name)
            .setCharacteristic(this.hap.Characteristic.Model, accessory.context.model)
            .setCharacteristic(this.hap.Characteristic.SerialNumber, accessory.context.deviceID);
    }
    async getDeviceLogSettings(device) {
        if (this.platform.debugMode) {
            this.deviceLogging = this.accessory.context.logging = 'debugMode';
            this.debugWarnLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Debug Mode Logging: ${this.deviceLogging}`);
        }
        else if (device.logging) {
            this.deviceLogging = this.accessory.context.logging = device.logging;
            this.debugWarnLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Device Config Logging: ${this.deviceLogging}`);
        }
        else if (this.config.options?.logging) {
            this.deviceLogging = this.accessory.context.logging = this.config.logging;
            this.debugWarnLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Platform Config Logging: ${this.deviceLogging}`);
        }
        else {
            this.deviceLogging = this.accessory.context.logging = 'standard';
            this.debugWarnLog(`${this.device.deviceClass}: ${this.accessory.displayName} Logging Not Set, Using: ${this.deviceLogging}`);
        }
    }
    async getDeviceRateSettings(device) {
        // refreshRate
        if (device.thermostat?.roomsensor?.refreshRate) {
            this.deviceRefreshRate = device.thermostat.roomsensor.refreshRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Room Sensor Config Refresh Rate: ${this.deviceRefreshRate}`);
        }
        else if (device.thermostat?.roompriority?.refreshRate) {
            this.deviceRefreshRate = device.thermostat.roompriority.refreshRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Room Priority Config Refresh Rate: ${this.deviceRefreshRate}`);
        }
        else if (device.refreshRate) {
            this.deviceRefreshRate = this.accessory.context.deviceRefreshRate = device.refreshRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Device Config Refresh Rate: ${this.deviceRefreshRate}`);
        }
        else {
            this.deviceRefreshRate = this.config.options?.refreshRate ?? 120;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Platform Config Refresh Rate: ${this.deviceRefreshRate}`);
        }
        this.accessory.context.deviceRefreshRate = this.deviceRefreshRate;
        // pushRate
        if (device.thermostat?.roomsensor?.pushRate) {
            this.devicePushRate = device.thermostat.roomsensor.pushRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Room Sensor Config Push Rate: ${this.devicePushRate}`);
        }
        else if (device.thermostat?.roompriority?.pushRate) {
            this.devicePushRate = device.thermostat.roompriority.pushRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Room Priority Config Push Rate: ${this.devicePushRate}`);
        }
        else if (device.pushRate) {
            this.devicePushRate = device.pushRate;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Device Config Push Rate: ${this.devicePushRate}`);
        }
        else {
            this.devicePushRate = this.config.options?.pushRate ?? 0.1;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Platform Push Rate: ${this.devicePushRate}`);
        }
        this.accessory.context.devicePushRate = this.devicePushRate;
    }
    async getDeviceRetry(device) {
        if (device.maxRetries) {
            this.deviceMaxRetries = device.maxRetries;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Using Device Max Retries: ${this.deviceMaxRetries}`);
        }
        else {
            this.deviceMaxRetries = 5; // Maximum number of retries
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Max Retries Not Set, Using: ${this.deviceMaxRetries}`);
        }
        if (device.delayBetweenRetries) {
            this.deviceDelayBetweenRetries = device.delayBetweenRetries * 1000;
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName}`
                + ` Using Device Delay Between Retries: ${this.deviceDelayBetweenRetries}`);
        }
        else {
            this.deviceDelayBetweenRetries = 3000; // Delay between retries in milliseconds
            this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Delay Between Retries Not Set,`
                + ` Using: ${this.deviceDelayBetweenRetries}`);
        }
    }
    async getDeviceConfigSettings(device) {
        const deviceConfig = {};
        if (device.logging !== 'standard') {
            deviceConfig['logging'] = device.logging;
        }
        if (device.external === true) {
            deviceConfig['external'] = device.external;
        }
        if (device.refreshRate !== 0) {
            deviceConfig['refreshRate'] = device.refreshRate;
        }
        if (device.pushRate !== 0) {
            deviceConfig['pushRate'] = device.pushRate;
        }
        if (device.retry === true) {
            deviceConfig['retry'] = device.retry;
        }
        if (device.maxRetries !== 0) {
            deviceConfig['maxRetries'] = device.maxRetries;
        }
        if (device.delayBetweenRetries !== 0) {
            deviceConfig['delayBetweenRetries'] = device.delayBetweenRetries;
        }
        let thermostatConfig = {};
        if (device.thermostat) {
            thermostatConfig = device.thermostat;
        }
        let leaksensorConfig = {};
        if (device.leaksensor) {
            leaksensorConfig = device.leaksensor;
        }
        let valveConfig = {};
        if (device.valve) {
            valveConfig = device.valve;
        }
        const config = Object.assign({}, deviceConfig, thermostatConfig, leaksensorConfig, valveConfig);
        if (Object.entries(config).length !== 0) {
            this.debugSuccessLog(`${this.device.deviceClass}: ${this.accessory.displayName} Config: ${JSON.stringify(config)}`);
        }
    }
    async getDeviceContext(accessory, device, sensorAccessory) {
        if (sensorAccessory?.accessoryAttribute) {
            accessory.context.name = sensorAccessory.accessoryAttribute.name;
            accessory.context.model = sensorAccessory.accessoryAttribute.model;
            accessory.context.deviceID = sensorAccessory.accessoryAttribute.serialNumber;
            accessory.context.deviceType = sensorAccessory.accessoryAttribute.type;
        }
        else if (device.deviceClass) {
            accessory.context.name = device.userDefinedDeviceName ?? device.name;
            accessory.context.model = device.deviceClass ?? device.deviceModel;
            accessory.context.deviceID = device.deviceID;
            accessory.context.deviceType = device.deviceType;
        }
        // Firmware Version
        let deviceFirmwareVersion;
        if (sensorAccessory?.accessoryAttribute.softwareRevision) {
            deviceFirmwareVersion = sensorAccessory.accessoryAttribute.softwareRevision;
        }
        else if (device.firmware) {
            deviceFirmwareVersion = device.firmware;
            this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 1 FirmwareRevision: ${device.firmware}`);
        }
        else if (device.firmwareVersion) {
            deviceFirmwareVersion = device.firmwareVersion;
            this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 2 FirmwareRevision: ${device.firmwareVersion}`);
        }
        else if (device.thermostatVersion) {
            deviceFirmwareVersion = device.thermostatVersion;
            this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 3 FirmwareRevision: ${device.thermostatVersion}`);
        }
        else if (accessory.context.deviceVersion) {
            deviceFirmwareVersion = accessory.context.deviceVersion;
            this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 4 FirmwareRevision: ${accessory.context.deviceVersion}`);
        }
        else {
            deviceFirmwareVersion = this.platform.version ?? '0.0.0';
            if (this.platform.version) {
                this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 5 FirmwareRevision: ${this.platform.version}`);
            }
            else {
                this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} 6 FirmwareRevision: ${deviceFirmwareVersion}`);
            }
        }
        const version = deviceFirmwareVersion.toString();
        this.debugLog(`${this.device.deviceType}: ${accessory.displayName} Firmware Version: ${version?.replace(/^V|-.*$/g, '')}`);
        let deviceVersion;
        if (version?.includes('.') === false) {
            const replace = version?.replace(/^V|-.*$/g, '');
            const match = replace?.match(/.{1,1}/g);
            const validVersion = match?.join('.');
            deviceVersion = validVersion ?? '0.0.0';
        }
        else {
            deviceVersion = version?.replace(/^V|-.*$/g, '') ?? '0.0.0';
        }
        accessory
            .getService(this.hap.Service.AccessoryInformation)
            .setCharacteristic(this.hap.Characteristic.HardwareRevision, deviceVersion)
            .setCharacteristic(this.hap.Characteristic.SoftwareRevision, deviceVersion)
            .setCharacteristic(this.hap.Characteristic.FirmwareRevision, deviceVersion)
            .getCharacteristic(this.hap.Characteristic.FirmwareRevision)
            .updateValue(deviceVersion);
        accessory.context.deviceVersion = deviceVersion;
        this.debugSuccessLog(`${device.deviceType}: ${accessory.displayName} deviceVersion: ${accessory.context.deviceVersion}`);
    }
    async statusCode(statusCode, action) {
        switch (statusCode) {
            case 200:
                this.debugLog(`${this.device.deviceClass}: ${this.accessory.displayName} Standard Response, statusCode: ${statusCode}, Action: ${action}`);
                break;
            case 400:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Bad Request, statusCode: ${statusCode}, Action: ${action}`);
                break;
            case 401:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Unauthorized, statusCode: ${statusCode}, Action: ${action}`);
                break;
            case 403:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Forbidden,	The request has been authenticated but does not `
                    + `have appropriate permissions, or a requested resource is not found, statusCode: ${statusCode}`);
                break;
            case 404:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Not Found, statusCode: ${statusCode}, Action: ${action}`);
                break;
            case 429:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Too Many Requests, statusCode: ${statusCode}, Action: ${action}`);
                break;
            case 500:
                this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} Internal Server Error (Meater Server), statusCode: ${statusCode}, `
                    + `Action: ${action}`);
                break;
            default:
                this.infoLog(`${this.device.deviceClass}: ${this.accessory.displayName} Unknown statusCode: ${statusCode}, `
                    + `Action: ${action}, Report Bugs Here: https://bit.ly/homebridge-resideo-bug-report`);
        }
    }
    async resideoAPIError(e, action) {
        if (e.message.includes('400')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Bad Request`);
            this.debugLog('The client has issued an invalid request. This is commonly used to specify validation errors in a request payload.');
        }
        else if (e.message.includes('401')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Unauthorized Request`);
            this.debugLog('Authorization for the API is required, but the request has not been authenticated.');
        }
        else if (e.message.includes('403')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Forbidden Request`);
            this.debugLog('The request has been authenticated but does not have appropriate permissions, or a requested resource is not found.');
        }
        else if (e.message.includes('404')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Requst Not Found`);
            this.debugLog('Specifies the requested path does not exist.');
        }
        else if (e.message.includes('406')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Request Not Acceptable`);
            this.debugLog('The client has requested a MIME type via the Accept header for a value not supported by the server.');
        }
        else if (e.message.includes('415')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Unsupported Requst Header`);
            this.debugLog('The client has defined a contentType header that is not supported by the server.');
        }
        else if (e.message.includes('422')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Unprocessable Entity`);
            this.debugLog('The client has made a valid request, but the server cannot process it.' +
                ' This is often used for APIs for which certain limits have been exceeded.');
        }
        else if (e.message.includes('429')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Too Many Requests`);
            this.debugLog('The client has exceeded the number of requests allowed for a given time window.');
        }
        else if (e.message.includes('500')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action}, Internal Server Error`);
            this.debugLog('An unexpected error on the SmartThings servers has occurred. These errors should be rare.');
        }
        else {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to ${action},`);
        }
        if (this.deviceLogging.includes('debug')) {
            this.errorLog(`${this.device.deviceClass}: ${this.accessory.displayName} failed to pushChanges, Error Message: ${JSON.stringify(e.message)}`);
        }
    }
    /**
     * Logging for Device
     */
    infoLog(...log) {
        if (this.enablingDeviceLogging()) {
            this.log.info(String(...log));
        }
    }
    successLog(...log) {
        if (this.enablingDeviceLogging()) {
            this.log.success(String(...log));
        }
    }
    debugSuccessLog(...log) {
        if (this.enablingDeviceLogging()) {
            if (this.deviceLogging?.includes('debug')) {
                this.log.success('[DEBUG]', String(...log));
            }
        }
    }
    warnLog(...log) {
        if (this.enablingDeviceLogging()) {
            this.log.warn(String(...log));
        }
    }
    debugWarnLog(...log) {
        if (this.enablingDeviceLogging()) {
            if (this.deviceLogging?.includes('debug')) {
                this.log.warn('[DEBUG]', String(...log));
            }
        }
    }
    errorLog(...log) {
        if (this.enablingDeviceLogging()) {
            this.log.error(String(...log));
        }
    }
    debugErrorLog(...log) {
        if (this.enablingDeviceLogging()) {
            if (this.deviceLogging?.includes('debug')) {
                this.log.error('[DEBUG]', String(...log));
            }
        }
    }
    debugLog(...log) {
        if (this.enablingDeviceLogging()) {
            if (this.deviceLogging === 'debug') {
                this.log.info('[DEBUG]', String(...log));
            }
            else {
                this.log.debug(String(...log));
            }
        }
    }
    enablingDeviceLogging() {
        return this.deviceLogging.includes('debug') || this.deviceLogging === 'standard';
    }
}
//# sourceMappingURL=device.js.map